#pragma once

#include <stdlib.h>
#include "signer.h"
#include <iostream>
#include <atlstr.h>

#include "WebsocketClient.h"
#include <fstream>
#include "string"
#include <vector>
#include "set"
#include "AuthInfo.h"
#include "RasrRequest.h"
#include "HttpConfig.h"
#include "map"
#include "RasrListener.h"


class RasrClient {

private:
    AuthInfo authInfo;
    WebsocketClient ws;
    HttpConfig httpConfig;
    WebsocketService::ptr websocketServicePtr;

    void Connect(std::string api);
    bool CheckStart();
    bool CheckBinary();
    bool CheckEnd();
    bool CheckConnect();
    void WaitStatus(std::set<WebsocketStatus> targetStatuses, int maxWaitTime);
public:
    RasrClient(AuthInfo authinfo, WebsocketService::ptr servicePtr, HttpConfig config) {
        authInfo = authinfo;
        httpConfig = config;
        websocketServicePtr = servicePtr;
        ws.setWebsocketServicePtr(websocketServicePtr);
		websocketServicePtr -> SetListenerType("rasr");
    }

    ~RasrClient(void) {};

    void ShortStreamConnect();

    void ContinueStreamConnect();

    void SentenceStreamConnect();

    void SendStart(RasrRequest request);

    void SendEnd();

    void SendBinary(unsigned char *buff, size_t size);

    void SendBinary(unsigned char *buff, size_t size, int byteLen, int sleepTime);

    void Close();


};

