//
// Created by z00500078 on 2022/5/17.
//
#pragma once
#ifndef HUAWEICLOUD_CPP_SDK_SIS_CLION_WEBSOCKETSERVICE_H
#define HUAWEICLOUD_CPP_SDK_SIS_CLION_WEBSOCKETSERVICE_H

#include <websocketpp/config/asio_client.hpp>
#include <websocketpp/client.hpp>

#include <websocketpp/common/thread.hpp>
#include <websocketpp/common/memory.hpp>
#include "string"
#include "WebsocketStatus.h"
#include "RasrListener.h"
#include "RttsListener.h"
#include "IoUtil.h"

using websocketpp::lib::bind;
typedef websocketpp::client<websocketpp::config::asio_tls_client> client;
typedef websocketpp::lib::shared_ptr<websocketpp::lib::asio::ssl::context> context_ptr;
typedef websocketpp::config::asio_client::message_type::ptr message_ptr;

typedef std::function<void()> OnConnectFunc;
typedef std::function<void()> OnCloseFunc;
typedef std::function<void(const std::string & text)> OnStartFunc;
typedef std::function<void(const std::string & text)> OnEndFunc;
typedef std::function<void(const std::string & text)> OnEventFunc;
typedef std::function<void(const std::string & text)> OnRespFunc;
typedef std::function<void(const std::string & text)> OnErrorFunc;
typedef std::function<void(const std::string & binaryData)> OnBinaryFunc;

class WebsocketService {
public:
    typedef websocketpp::lib::shared_ptr<WebsocketService> ptr;

    void OnOpen(client *c, websocketpp::connection_hdl hdl);

    void OnFail(client *c, websocketpp::connection_hdl hdl);

    void OnClose(client *c, websocketpp::connection_hdl hdl);

    void OnMessage(websocketpp::connection_hdl, client::message_ptr msg);


    WebsocketStatus GetStatus() {
        return status;
    }

    void SetStatus(WebsocketStatus newStatus) {
        status = newStatus;
    }

    void SetOnConnectFunc(OnConnectFunc func) {
        onConnectFunc  = func;
    }

    void SetOnEndFunc(OnEndFunc func) {
        onEndFunc = func;
    }

    void SetOnErrorFunc(OnErrorFunc func) {
        onErrorFunc = func;
    }

    void SetOnRespFunc(OnRespFunc func) {
        onRespFunc = func;
    }

    void SetOnCloseFunc(OnCloseFunc func) {
        onCloseFunc = func;
    }

    void SetOnEventFunc(OnEventFunc func) {
        onEventFunc = func;
    }

    void SetOnStartFunc(OnStartFunc func) {
        onStartFunc = func;
    }

	void SetOnBinaryFunc(OnBinaryFunc func) {
		onBinaryFunc = func;
	}

	void SetListenerType(std::string ltype) {
		listenerType = ltype;
	}

	void SetRttsListener(RttsListener listener) {
		rttsListener = listener;
	}

	void SetRasrListener(RasrListener listener) {
		rasrListener = listener;
	}

private:
    WebsocketStatus status = WB_PRE_START;
    RasrListener rasrListener;
	RttsListener rttsListener;
	std::string listenerType = "rasr";
    OnConnectFunc onConnectFunc  = nullptr;
    OnStartFunc onStartFunc = nullptr;
    OnRespFunc onRespFunc = nullptr;
    OnEndFunc onEndFunc = nullptr;
    OnCloseFunc onCloseFunc = nullptr;
    OnErrorFunc onErrorFunc = nullptr;
    OnEventFunc onEventFunc = nullptr;
	OnBinaryFunc onBinaryFunc = nullptr;
    void ProcessMessage(std::string message);
    void ProcessConeect();
    void ProcessStart(std::string text);
    void ProcessEnd(std::string text);
    void ProcessError(std::string text);
    void ProcessClose();
    void ProcessResp(std::string text);
    void ProcessEvent(std::string text);
	void ProcessBinary(std::string binaryData);

};

#endif //HUAWEICLOUD_CPP_SDK_SIS_CLION_WEBSOCKETSERVICE_H
