﻿#include "RasrClient.h"
#include "AuthUtil.h"
#include "thread"
#include "chrono"

const int BYTE_LEN = 3200;

bool RasrClient::CheckStart() {
	WebsocketStatus status = websocketServicePtr->GetStatus();
	if (status == WB_CONNECT || status == WB_END) {
		return true;
	}
	return false;
}

bool RasrClient::CheckBinary() {
	WebsocketStatus status = websocketServicePtr->GetStatus();
	if (status == WB_START) {
		return true;
	}
	return false;
}

bool RasrClient::CheckEnd() {
	WebsocketStatus status = websocketServicePtr->GetStatus();
	if (status == WB_START) {
		return true;
	}
	return false;
}

bool RasrClient::CheckConnect() {
	WebsocketStatus status = websocketServicePtr->GetStatus();
	if (status == WB_PRE_START || status == WB_CLOSE) {
		return true;
	}
	if (status == WB_ERROR) {
		Close();
		return true;
	}
	return false;
}

void RasrClient::WaitStatus(std::set<WebsocketStatus> targetStatuses, int maxWaitTime) {
	int sleepTime = 100;
	int sleepCount = (int)(maxWaitTime / sleepTime);
	int count = 0;
	while (count < sleepCount) {
		count += 1;
		if (targetStatuses.find(websocketServicePtr->GetStatus()) != targetStatuses.end()) {
			break;
		}
		std::this_thread::sleep_for(std::chrono::milliseconds(sleepTime));
	}

	if (targetStatuses.find(websocketServicePtr->GetStatus()) == targetStatuses.end()) {
		std::cout << "wait status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " failed" << std::endl;
	}
}

void RasrClient::Close() {
	ws.Close("normal");
	std::set<WebsocketStatus> targetStatuses{ WB_ERROR, WB_CLOSE };
	WaitStatus(targetStatuses, httpConfig.GetReadTimeout());

}

void RasrClient::ShortStreamConnect() {
	std::string api = "/v1/" + authInfo.GetProjectId() + "/rasr/short-stream";
	Connect(api);

}

void RasrClient::SendStart(RasrRequest request) {
	if (CheckStart()) {
		websocketServicePtr->SetStatus(WB_BLOCKING);
		ws.SendTxt(request.ConstructParams());
		std::set<WebsocketStatus> targetStatuses{ WB_START, WB_ERROR, WB_CLOSE };
		WaitStatus(targetStatuses, httpConfig.GetReadTimeout());
	}
	else {
		std::cout << "status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " can't send start" << std::endl;
	}

}


void RasrClient::ContinueStreamConnect() {
	std::string api = "/v1/" + authInfo.GetProjectId() + "/rasr/continue-stream";
	Connect(api);

}


void RasrClient::SentenceStreamConnect() {
	std::string api = "/v1/" + authInfo.GetProjectId() + "/rasr/sentence-stream";
	Connect(api);
}

void RasrClient::SendEnd() {
	if (CheckEnd()) {
		websocketServicePtr->SetStatus(WB_BLOCKING);
		ws.SendTxt("{\"command\": \"END\", \"cancel\": \"false\"}");
		std::set<WebsocketStatus> targetStatuses{ WB_END, WB_ERROR, WB_CLOSE };
		WaitStatus(targetStatuses, httpConfig.GetReadTimeout());
	}
	else {
		std::cout << "status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " can't send end" << std::endl;
	}

}

void RasrClient::SendBinary(unsigned char *buff, size_t size) {
	SendBinary(buff, size, BYTE_LEN, 0);

}

void RasrClient::SendBinary(unsigned char *buff, size_t size, int byteLen, int sleepTime) {
	if (CheckBinary()) {
		if (size < 20000 || size < byteLen) {
			ws.SendBinary(buff, size);
			return;
		}
		int index = 0;
		int tempSize = byteLen;
		while (index < size) {
			if (index + byteLen > size) {
				tempSize = size - index;
			}
			else {
				tempSize = byteLen;
			}
			ws.SendBinary(buff + index, tempSize);
			std::this_thread::sleep_for(std::chrono::milliseconds(sleepTime));
			index += byteLen;
		}
	}
	else {
		std::cout << "status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " can't send binary data" << std::endl;
	}


}


void RasrClient::Connect(std::string api) {
	if (CheckConnect()) {
		std::string url = "wss://" + authInfo.GetEndpoint() + api;
		std::map<std::string, std::string> headers = SignHeaders(authInfo, api, "", "", "GET");
		websocketServicePtr->SetStatus(WB_BLOCKING);
		ws.Connect(url, headers);
	}
	else {
		std::cout << "status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " can't Connect" << std::endl;
	}

	std::set<WebsocketStatus> targetStatuses{ WB_CONNECT, WB_ERROR, WB_CLOSE };
	WaitStatus(targetStatuses, httpConfig.GetConnectTimeout());
}

