#include "RttsClient.h"
#include "AuthUtil.h"
#include "thread"
#include "chrono"

const int BYTE_LEN = 3200;

void RttsClient::Synthesis(RttsRequest request) {
	Connect();
	SendRequest(request);
	Close();
}

bool RttsClient::CheckSend() {
	WebsocketStatus status = websocketServicePtr->GetStatus();
	if (status == WB_CONNECT) {
		return true;
	}
	return false;
}


bool RttsClient::CheckConnect() {
	WebsocketStatus status = websocketServicePtr->GetStatus();
	if (status == WB_PRE_START || status == WB_CLOSE) {
		return true;
	}
	if (status == WB_ERROR) {
		Close();
		return true;
	}
	return false;
}

void RttsClient::WaitStatus(std::set<WebsocketStatus> targetStatuses, int maxWaitTime) {
	int sleepTime = 100;
	int sleepCount = (int)(maxWaitTime / sleepTime);
	int count = 0;
	while (count < sleepCount) {
		count += 1;
		if (targetStatuses.find(websocketServicePtr->GetStatus()) != targetStatuses.end()) {
			break;
		}
		std::this_thread::sleep_for(std::chrono::milliseconds(sleepTime));
	}

	if (targetStatuses.find(websocketServicePtr->GetStatus()) == targetStatuses.end()) {
		std::cout << "wait status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " failed" << std::endl;
	}
}

void RttsClient::Close() {
	ws.Close("normal");
	std::set<WebsocketStatus> targetStatuses{ WB_ERROR, WB_CLOSE };
	WaitStatus(targetStatuses, httpConfig.GetReadTimeout());

}

void RttsClient::SendRequest(RttsRequest request) {
	if (CheckSend()) {
		websocketServicePtr->SetStatus(WB_BLOCKING);
		ws.SendTxt(request.ConstructParams());
		std::set<WebsocketStatus> targetStatuses{ WB_END, WB_ERROR, WB_CLOSE };
		WaitStatus(targetStatuses, httpConfig.GetReadTimeout());
	}
	else {
		std::cout << "status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " can't send request" << std::endl;
	}

}


void RttsClient::Connect() {
	std::string api = "/v1/" + authInfo.GetProjectId() + "/rtts";
	if (CheckConnect()) {
		std::string url = "wss://" + authInfo.GetEndpoint() + api;
		std::map<std::string, std::string> headers = SignHeaders(authInfo, api, "", "", "GET");
		websocketServicePtr->SetStatus(WB_BLOCKING);
		ws.Connect(url, headers);
	}
	else {
		std::cout << "status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " can't Connect" << std::endl;
	}

	std::set<WebsocketStatus> targetStatuses{ WB_CONNECT, WB_ERROR, WB_CLOSE };
	WaitStatus(targetStatuses, httpConfig.GetConnectTimeout());
}

