#include "WebsocketClient.h"

context_ptr OnTlsInit() {
	context_ptr ctx = websocketpp::lib::make_shared<boost::asio::ssl::context>(boost::asio::ssl::context::sslv23);
	try {
		ctx->set_options(boost::asio::ssl::context::default_workarounds |
			boost::asio::ssl::context::no_sslv2 |
			boost::asio::ssl::context::no_sslv3 |
			boost::asio::ssl::context::single_dh_use);
	}
	catch (std::exception &e) {
		std::cout << "ssl occurs " << e.what() << std::endl;
	}
	return ctx;
}

WebsocketClient::WebsocketClient() {
	mClient.clear_access_channels(websocketpp::log::alevel::all);
	mClient.clear_error_channels(websocketpp::log::elevel::all);

	mClient.init_asio();
	mClient.start_perpetual();
	mThread = websocketpp::lib::make_shared<websocketpp::lib::thread>(&client::run, &mClient);
}

WebsocketClient::~WebsocketClient() {
	mClient.stop_perpetual();

	if (websocketServicePtr->GetStatus() != WB_CLOSE && websocketServicePtr->GetStatus() != WB_PRE_START) {
		std::cout << "now status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " have not been closed, it will be forced closed"
			<< std::endl;
		websocketpp::lib::error_code ec;
		mClient.close(hdl, websocketpp::close::status::going_away, "", ec); // ر

		if (ec) {
			std::cout << "> Error forced Close: " << ec.message() << std::endl;
		}
	}

	mThread->join();
}


bool WebsocketClient::Connect(std::string const &url, std::map<std::string, std::string> headers) {
	mClient.set_tls_init_handler(bind(&OnTlsInit));

	//mClient.set_tcp_init_handler();
	websocketpp::lib::error_code ec;

	// create shared Connect ptr
	client::connection_ptr con = mClient.get_connection(url, ec);
	hdl = con->get_handle();


	if (ec) {
		std::cout << "> Connect initialization error: " << ec.message() << std::endl;
		return false;
	}

	// register OnOpen
	con->set_open_handler(websocketpp::lib::bind(
		&WebsocketService::OnOpen,
		websocketServicePtr,
		&mClient,
		websocketpp::lib::placeholders::_1
	));

	// register OnFail
	con->set_fail_handler(websocketpp::lib::bind(
		&WebsocketService::OnFail,
		websocketServicePtr,
		&mClient,
		websocketpp::lib::placeholders::_1
	));

	// register OnClose
	con->set_close_handler(websocketpp::lib::bind(
		&WebsocketService::OnClose,
		websocketServicePtr,
		&mClient,
		websocketpp::lib::placeholders::_1
	));

	// register OnMessage
	con->set_message_handler(websocketpp::lib::bind(
		&WebsocketService::OnMessage,
		websocketServicePtr,
		websocketpp::lib::placeholders::_1,
		websocketpp::lib::placeholders::_2
	));

	for (std::map<std::string, std::string>::iterator iteratorMap = headers.begin(); iteratorMap != headers.end(); iteratorMap++) {
		con->append_header(iteratorMap->first, iteratorMap->second);
	}

	// execute Connect
	mClient.connect(con);


	return true;
}

bool WebsocketClient::Close(std::string reason) {
	websocketpp::lib::error_code ec;
	int close_code = websocketpp::close::status::normal;
	if (websocketServicePtr->GetStatus() != WB_CLOSE && websocketServicePtr->GetStatus() != WB_PRE_START) {
		mClient.close(hdl, close_code, reason, ec);
		if (ec) {
			std::cout << "> Error Close: " << ec.message() << std::endl;
			return false;
		}
		return true;
	}
	std::cout << "status " << WebsocketStatusToStr(websocketServicePtr->GetStatus()) << " can't be closed" << std::endl;
	return false;

}

bool WebsocketClient::SendTxt(std::string message) {

	websocketpp::lib::error_code ec;

	mClient.send(hdl, message,
		websocketpp::frame::opcode::text, ec);
	if (ec) {
		std::cout << "> Error sending message: " << ec.message() << std::endl;
		return false;
	}

	mClient.run_one();
	return true;
}

bool WebsocketClient::SendBinary(unsigned char *buff, size_t size) {
	websocketpp::lib::error_code ec;

	// continue send
	mClient.send(hdl, buff, size, websocketpp::frame::opcode::binary, ec);
	if (ec) {
		std::cout << "> Error sending message: " << ec.message() << std::endl;
		return false;
	}


	return true;
}
