//
// Created by z00500078 on 2022/5/17.
//
#include "WebsocketService.h"


void WebsocketService::OnOpen(client *c, websocketpp::connection_hdl hdl) {
	ProcessConeect();
}

void WebsocketService::OnClose(client *c, websocketpp::connection_hdl hdl) {
	ProcessClose();
}

void WebsocketService::OnFail(client *c, websocketpp::connection_hdl hdl) {
	status = WB_ERROR; // avoid next step failed
	client::connection_ptr con = c->get_con_from_hdl(hdl);
	std::string errorMsg = con->get_ec().message();
	ProcessError(errorMsg);
}

void WebsocketService::OnMessage(websocketpp::connection_hdl, client::message_ptr msg) {
	if (msg->get_opcode() == websocketpp::frame::opcode::text) {
		std::string message = msg->get_payload();
		ProcessMessage(message);
	} else if (msg->get_opcode() == websocketpp::frame::opcode::binary) {
		std::string binaryData = msg->get_payload();
		ProcessBinary(binaryData);
	} else {
		std::string errorMsg = "receive data type  is wrong";
		ProcessError(errorMsg);
	}
}

void WebsocketService::ProcessMessage(std::string message) {
	std::string suffix = "resp_type\":\"";
	if (message.find(suffix + "START") != std::string::npos) {
		ProcessStart(message);
	}
	else if (message.find(suffix + "END") != std::string::npos) {
		ProcessEnd(message);
	}
	else if (message.find(suffix + "EVENT") != std::string::npos) {
		ProcessEvent(message);
	}
	else if (message.find(suffix + "ERROR") != std::string::npos || message.find(suffix + "FATAL_ERROR") != std::string::npos) {
		ProcessError(message);
	}
	else if (message.find(suffix + "RESULT") != std::string::npos) {
		ProcessResp(message);
	}
	else {
		ProcessError(message);
		return;
	}


}

void WebsocketService::ProcessBinary(std::string binaryData) {
	if (onBinaryFunc != nullptr) {
		onBinaryFunc(binaryData);
	} else {
		rttsListener.OnResp(binaryData);
	}
}
void WebsocketService::ProcessConeect() {
	status = WB_CONNECT;
	if (onConnectFunc != nullptr) {
		onConnectFunc();
	}
	else if (listenerType.compare("rasr") == 0) {
		rasrListener.OnConnect();
	} 
	else {
		rttsListener.OnConnect();
	}
}

void WebsocketService::ProcessStart(std::string text) {
	status = WB_START;
	if (onStartFunc != nullptr) {
		onStartFunc(text);
	}
	else if (listenerType.compare("rasr") == 0) {
		rasrListener.OnStart(text);
	} 
	else {
		rttsListener.OnStart(text);
	}
}

void WebsocketService::ProcessEnd(std::string text) {
	status = WB_END;
	if (onEndFunc != nullptr) {
		onEndFunc(text);
	}
	else if (listenerType.compare("rasr") == 0) {
		rasrListener.OnEnd(text);
	} 
	else {
		rttsListener.OnEnd(text);
	}
}

void WebsocketService::ProcessResp(std::string text) {
	if (onRespFunc != nullptr) {
		onRespFunc(text);
	}
	else   {
		rasrListener.OnResp(text);
	}
	
}

void WebsocketService::ProcessEvent(std::string text) {
	if (onEventFunc != nullptr) {
		onEventFunc(text);
	}
	else {
		rasrListener.OnEvent(text);
	}
}

void WebsocketService::ProcessClose() {
	status = WB_CLOSE;
	if (onCloseFunc != nullptr) {
		onCloseFunc();
	}
	else  if (listenerType.compare("rasr") == 0) {
		rasrListener.OnClose();
	}
	else {
		rttsListener.OnClose();
	}
}

void WebsocketService::ProcessError(std::string text) {
	std::cout << "websocket error: " << text << std::endl;
	status = WB_ERROR;
	if (onErrorFunc != nullptr) {
		onErrorFunc(text);
	}
	else  if (listenerType.compare("rasr") == 0) {
		rasrListener.OnError(text);
	}
	else {
		rttsListener.OnError(text);
	}
}