#include <iostream>

#include "RasrClient.h"
#include "RasrRequest.h"
#include "IoUtil.h"


void OnRasrConnect() {
	std::cout << "now rasr client Connect success" << std::endl;
}

void OnRasrStart(std::string text) {
	std::cout << "now rasr client receive start response: " << text << std::endl;
}

void OnRasrResp(std::string text) {
	// text encoded by utf-8 contains chinese character, which will cause error code. So we should convert to ansi
	// cout << "rasr receive " << text << endl;
	std::cout << "now rasr client receive " << Utf8ToAnsi(text) << std::endl;
}

void OnRasrEnd(std::string text) {
	std::cout << "now rasr client receive end response: " << text << std::endl;
}


void OnRasrClose() {
	std::cout << "now rasr client receive Close" << std::endl;
}

void OnRasrError(std::string text) {
	std::cout << "now rasr client receive error: " << text << std::endl;
}

void OnRasrEvent(std::string text) {
	std::cout << "now rasr client receive event: " << text << std::endl;
}

void RasrTest() {
	// 1. config parameter
	// 1.1 init authInfo
	// 认证用的ak和sk硬编码到代码中或者明文存储都有很大的安全风险，建议在配置文件或者环境变量中密文存放，使用时解密，确保安全；
    // 本示例以ak和sk保存在环境变量中来实现身份验证为例，运行本示例前请先在本地环境中设置环境变量HUAWEICLOUD_SDK_AK和HUAWEICLOUD_SDK_SK
	std::string ak = GetEnv("HUAWEICLOUD_SDK_AK");
	std::string sk = GetEnv("HUAWEICLOUD_SDK_SK");
	std::string region = "";
	std::string projectId = "";
	AuthInfo authInfo(ak, sk, region, projectId);

	// 1.2 config Connect parameter
	HttpConfig httpConfig;
	httpConfig.SetReadTimeout(20000);
	httpConfig.SetConnectTimeout(20000);

	// 1.3 config callback, callback function are optional, if not set, it will use function in RasrListener
	WebsocketService::ptr websocketServicePtr = websocketpp::lib::make_shared<WebsocketService>();
	websocketServicePtr->SetOnConnectFunc(OnRasrConnect);       // Connect success callback
	websocketServicePtr->SetOnStartFunc(OnRasrStart);				// receive start response callback
	websocketServicePtr->SetOnRespFunc(OnRasrResp);             // receive transcribe result callback
	websocketServicePtr->SetOnEndFunc(OnRasrEnd);               // receive end response callback
	websocketServicePtr->SetOnCloseFunc(OnRasrClose);           // Close callback
	websocketServicePtr->SetOnEventFunc(OnRasrEvent);           // receive event callback
	websocketServicePtr->SetOnErrorFunc(OnRasrError);           // receive error callback

	// 1.3 option, use RasrListener,  You can edit RasrListener.h to finish your own business
	// WebsocketService::ptr websocketServicePtr = websocketpp::lib::make_shared<WebsocketService>();
	// RasrListener rasrListener;
	// websocketServicePtr->SetRasrListener(rasrListener);


	// 1.4 config request parameter
	RasrRequest request("pcm16k16bit", "chinese_16k_general");
	request.SetIntermediateResult("no");


	// 2. init client
	RasrClient* rasrClient = new RasrClient(authInfo, websocketServicePtr, httpConfig);

	// 3. create connection :ContinueStreamConnect/ShortStreamConnect/SentenceStreamConnect
	rasrClient->ContinueStreamConnect();

	// 4. send start
	rasrClient->SendStart(request);

	// 5. send binary audio. (filePtr, fileLength, byteLen, SleepTime ). If the audio is generated by recording, then it should set sleep time 0.
	int fileLength;
	std::string filePath = "../sisCppSdkDemo/16k_pcm.wav";
	unsigned char* buff = ReadBinary(filePath, &fileLength);
	if (buff == nullptr) {
		std::cout << filePath << " read file failed";
		rasrClient->Close();
		delete rasrClient;
		return;
	}
	rasrClient->SendBinary(buff, fileLength, 3200, 50);

	// 6. send end
	rasrClient->SendEnd();

	// 7. close
	rasrClient->Close();

	delete[] buff;
	delete rasrClient;
}


